# -*- coding: utf-8 -*-
#
# Copyright (c) 2007 by H. Turgut Uyar <uyar@tekir.org>
#
# This file is part of Humphrey Videotheque.
#
# Humphrey Videotheque is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Humphrey Videotheque is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = """H. Turgut Uyar <uyar@tekir.org>"""
__docformat__ = "restructuredtext"


from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import StringField, StringWidget
from Products.Archetypes.atapi import IntegerField, IntegerWidget
from Products.Archetypes.atapi import FloatField, DecimalWidget
from Products.Archetypes.atapi import TextField, TextAreaWidget
from Products.Archetypes.atapi import ComputedField, ComputedWidget
from Products.Archetypes.atapi import ReferenceField
from Products.Archetypes.atapi import SelectionWidget
from Products.ATContentTypes.content.base import ATCTContent
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.Videotheque.config import PROJECTNAME
from Products.Videotheque.content import Videotheque
from Products.Videotheque.content.schemata import SingleLanguageSchema
from Products.Videotheque.content.schemata import MultipleLanguageSchema
from Products.Videotheque.content.schemata import SingleSubtitleSchema
from Products.Videotheque.content.schemata import MultipleSubtitleSchema
from Products.Videotheque.content.schemata import TVSystemSchema
from Products.Videotheque.widget import MovieBrowserWidget


MovieCopySchema = ATContentTypeSchema.copy() + Schema((
    ReferenceField(
        name="movie",
        required=True,
        multiValued=False,
        relationship="movie_copy",
        allowed_types=("Movie",),
        widget=MovieBrowserWidget(
            label_msgid="label_movie",
            label="Movie",
            description_msgid="help_movie_reference",
            description="Select the movie.",
            i18n_domain="videotheque"
            )
        ),

    StringField(
        name="identifier",
        widget=StringWidget(
            label_msgid="label_identifier",
            label="Identifier",
            description_msgid="help_identifier_copy",
            description="If there is more than one copy of the same movie on this collection item, you can use this field to distinguish this copy.",
            i18n_domain="videotheque"
            )
        ),

    ComputedField(
        name="title",
        accessor="Title",
        widget=ComputedWidget(
            label_msgid="label_title",
            modes=("view",),
            i18n_domain="plone"
            )
        ),

    StringField(
        name="aspect_ratio",
        vocabulary="_getAspectRatioOptions", enforceVocabulary=True,
        widget=SelectionWidget(
            label_msgid="label_aspect_ratio",
            label="Aspect Ratio",
            description_msgid="help_aspect_ratio_copy",
            description="Select the aspect ratio of this copy.",
            i18n_domain="videotheque"
            )
        ),

    IntegerField(
        name="runtime",
        widget=IntegerWidget(
            label_msgid="label_runtime",
            label="Runtime",
            description_msgid="help_runtime_copy",
            description="Enter the runtime of this copy, in minutes.",
            maxlength=3, size=3,
            i18n_domain="videotheque"
            )
        ),

    IntegerField(
        name="items",
        widget=IntegerWidget(
            label_msgid="label_items_copy",
            label="Number of Items",
            description_msgid="help_items_copy",
            description="Enter the number of items in this copy, i.e. how many discs, files etc.",
            size=3,
            i18n_domain="videotheque"
            )
        ),

    TextField(
        name="notes",
        searchable=True,
        widget=TextAreaWidget(
            label_msgid="label_notes",
            label="Notes",
            description_msgid="help_notes_copy",
            description="Enter your notes about this copy.",
            i18n_domain="videotheque"
            )
        ),
    ),)


MovieCopyDVDSchema = MovieCopySchema.copy() + MultipleLanguageSchema.copy() \
                   + MultipleSubtitleSchema.copy()


MovieCopyVCDSchema = MovieCopySchema.copy() + SingleLanguageSchema.copy() \
                   + SingleSubtitleSchema.copy()


MovieCopyFileSchema = MovieCopySchema.copy() + TVSystemSchema.copy() \
                    + MultipleLanguageSchema.copy() \
                    + MultipleSubtitleSchema.copy() + Schema((
    StringField(
        name="video_codec",
        vocabulary="_getVideoCodecOptions", enforceVocabulary=True,
        widget=SelectionWidget(
            label_msgid="label_video_codec",
            label="Video Codec",
            description_msgid="help_video_codec_file",
            description="Select the video codec of this movie file.",
            i18n_domain="videotheque"
            )
        ),

    StringField(
        name="audio_codec",
        vocabulary='_getAudioCodecOptions', enforceVocabulary=True,
        widget=SelectionWidget(
            label_msgid="label_audio_codec",
            label="Audio Codec",
            description_msgid="help_audio_codec_file",
            description="Select the audio codec of this movie file.",
            i18n_domain="videotheque"
            )
        ),

    FloatField(
        name="size",
        widget=DecimalWidget(
            label_msgid="label_size",
            label="Size",
            description_msgid="help_size_file",
            description="Enter the size of this movie file, in MB.",
            i18n_domain="videotheque"
            )
        ),
    ),)


MovieCopyTapeSchema = MovieCopySchema.copy() + SingleLanguageSchema.copy() \
                    + SingleSubtitleSchema.copy()


finalizeATCTSchema(MovieCopySchema)
finalizeATCTSchema(MovieCopyVCDSchema)
finalizeATCTSchema(MovieCopyDVDSchema)
finalizeATCTSchema(MovieCopyFileSchema)
finalizeATCTSchema(MovieCopyTapeSchema)


class MovieCopy(ATCTContent):
    """A copy of a movie."""

    __implements__ = (ATCTContent.__implements__,)

    schema = MovieCopySchema

    security = ClassSecurityInfo()

    security.declarePublic("Title")
    def Title(self):
        try:
            reference_tool = getToolByName(self, "reference_catalog")
            movie_references = reference_tool.getReferences(self, "movie_copy")
            movie = movie_references[0].getTargetObject()
            title = movie.Title()
        except:
            title = ""

        try:
            if self.identifier:
                title = "%s (%s)" % (title, self.identifier)
        except AttributeError:
            pass
        return title

    security.declarePublic("_getLanguageOptions")
    def _getLanguageOptions(self):
        options = ["None",]
        options.extend(self.videotheque_tool.getOptions("language_options"))
        return tuple(options)

    security.declarePublic("getParentIcon")
    def getParentIcon(self):
        parent = self.getParentNode()
        return parent.getIcon()

    security.declarePublic("getParentTitle")
    def getParentTitle(self):
        parent = self.getParentNode()
        return parent.Title()

    security.declarePublic("getVideotheque")
    def getVideotheque(self):
        """Return the videotheque object that this copy belongs to."""
        parent = self.getParentNode()
        while not isinstance(parent, Videotheque):
            parent = parent.getParentNode()
        return parent

    security.declarePublic("getStartupDirectory")
    def getStartupDirectory(self):
        dirs = []
        node = self
        while not isinstance(node, Videotheque):
            dirs.append("..")
            node = node.getParentNode()
        dirs.append(node.movies_folder_id)
        return '/'.join(dirs)

    security.declarePublic("_getTvSystemOptions")
    def _getTvSystemOptions(self):
        options = self.videotheque_tool.getOptions("tv_system_options")
        return tuple(options)

    security.declarePublic("_getAspectRatioOptions")
    def _getAspectRatioOptions(self):
        options = self.videotheque_tool.getOptions("aspect_ratio_options")
        return tuple(options)


class MovieCopyDVD(MovieCopy):
    """A movie copy on a DVD."""

    __implements__ = (MovieCopy.__implements__,)

    schema = MovieCopyDVDSchema

    _at_rename_after_creation = True

    security = ClassSecurityInfo()


class MovieCopyVCD(MovieCopy):
    """A movie copy on a VCD."""

    __implements__ = (MovieCopy.__implements__,)

    schema = MovieCopyVCDSchema

    _at_rename_after_creation = True

    security = ClassSecurityInfo()


class MovieCopyFile(MovieCopy):
    """A movie copy as a file."""

    __implements__ = (MovieCopy.__implements__,)

    schema = MovieCopyFileSchema

    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    security.declarePublic("_getVideoCodecOptions")
    def _getVideoCodecOptions(self):
        options = self.videotheque_tool.getOptions("video_codec_options")
        return tuple(options)

    security.declarePublic("_getAudioCodecOptions")
    def _getAudioCodecOptions(self):
        options = self.videotheque_tool.getOptions("audio_codec_options")
        return tuple(options)


class MovieCopyTape(MovieCopy):
    """A movie copy on a tape."""

    __implements__ = (MovieCopy.__implements__,)

    schema = MovieCopyTapeSchema

    _at_rename_after_creation = True

    security = ClassSecurityInfo()


registerATCT(MovieCopyDVD, PROJECTNAME)
registerATCT(MovieCopyVCD, PROJECTNAME)
registerATCT(MovieCopyFile, PROJECTNAME)
registerATCT(MovieCopyTape, PROJECTNAME)
