# -*- coding: utf-8 -*-
#
# Copyright (c) 2007 by H. Turgut Uyar <uyar@tekir.org>
#
# This file is part of Humphrey Videotheque.
#
# Humphrey Videotheque is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Humphrey Videotheque is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = """H. Turgut Uyar <uyar@tekir.org>"""
__docformat__ = "restructuredtext"


from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import LinesField
from Products.Archetypes.atapi import MultiSelectionWidget
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.folder import ATFolder
from Products.ATContentTypes.content.folder import ATFolderSchema
from Products.ATContentTypes.content.folder import ATBTreeFolder
from Products.ATContentTypes.content.folder import ATBTreeFolderSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.Videotheque.config import PROJECTNAME
from Products.Videotheque.permissions import ViewMovieCopy
from Products.Videotheque.permissions import AddMovie


VideothequeSchema = ATFolderSchema.copy() + Schema((
    LinesField(
        name="aka_countries",
        vocabulary="_getCountryOptions", enforceVocabulary=True,
        multiValued=True,
        widget=MultiSelectionWidget(
            label_msgid="label_aka_countries",
            label="Aka Countries",
            description_msgid="help_aka_countries_videotheque",
            description="Select the countries for getting the alternative titles of movies.",
            i18n_domain="videotheque"
            )
        ),
    ),)


VideothequeSectionSchema = ATBTreeFolderSchema.copy()


finalizeATCTSchema(VideothequeSchema, folderish=True)
finalizeATCTSchema(VideothequeSectionSchema, folderish=True)


class Videotheque(ATFolder):
    """A video collection."""

    __implements__ = (ATFolder.__implements__,)

    schema = VideothequeSchema

    _at_rename_after_creation = True

    movies_folder_id = "movies"
    movies_folder_title = "Movies"
    movie_prefix = "tt"

    security = ClassSecurityInfo()

    security.declarePrivate("initializeArchetype")
    def initializeArchetype(self, **kwargs):
        ATFolder.initializeArchetype(self, **kwargs)
        self.addMoviesFolder()

        self.setConstrainTypesMode(True)
        allowedTypes = ("VideothequeSection",)
        self.setLocallyAllowedTypes(allowedTypes)
        self.setImmediatelyAddableTypes(allowedTypes)

    security.declarePrivate("addMoviesFolder")
    def addMoviesFolder(self):
        """Create and return a default movies folder for this videotheque."""
        self.invokeFactory("Large Plone Folder", self.movies_folder_id)
        movies_folder = getattr(self, self.movies_folder_id)
        movies_folder.update(title=self.movies_folder_title)

        movies_folder.setConstrainTypesMode(True)
        allowedTypes = ("Movie", "Topic")
        movies_folder.setLocallyAllowedTypes(allowedTypes)
        movies_folder.setImmediatelyAddableTypes(("Movie",))
        return movies_folder

    security.declareProtected(AddMovie, "addMovie")
    def addMovie(self, movieid):
        """Add and return the movie with the given id to the movies folder."""
        movies_folder = getattr(self, self.movies_folder_id)
        videotheque_tool = getToolByName(self, "videotheque_tool")
        movie_info = videotheque_tool.getMovieInfoFromImdb(movieid,
                                                           self.aka_countries)

        objectid = self.movie_prefix + movieid
        movies_folder.invokeFactory("Movie", objectid)
        movie = getattr(movies_folder, objectid)
        movie.update(**movie_info)
        return movie

    security.declarePublic("_getCountryOptions")
    def _getCountryOptions(self):
        options = self.videotheque_tool.getOptions("country_options")
        return tuple(options)

    security.declareProtected(ViewMovieCopy, "getMovieReference")
    def getMovieReference(self, movieid):
        """Return a reference to the movie object with the given id.

        Create the movie in the movies folder if it is not already there."""
        try:
            movies_folder = getattr(self, self.movies_folder_id)
            objectid = self.movie_prefix + movieid
            movie = getattr(movies_folder, objectid)
        except AttributeError:
            movie = self.addMovie(movieid)
        return movie.UID()

    security.declarePublic("getMovie")
    def getMovie(self, movieid):
        try:
            movies_folder = getattr(self, self.movies_folder_id)
            objectid = self.movie_prefix + movieid
            movie = getattr(movies_folder, objectid)
            return movie
        except AttributeError:
            return None


class VideothequeSection(ATBTreeFolder):
    """A section in a video collection."""

    __implements__ = (ATBTreeFolder.__implements__,)

    schema = VideothequeSectionSchema

    _at_rename_after_creation = True

    security = ClassSecurityInfo()


registerATCT(Videotheque, PROJECTNAME)
registerATCT(VideothequeSection, PROJECTNAME)
