# -*- coding: utf-8 -*-
#
# Copyright (c) 2007 by H. Turgut Uyar <uyar@tekir.org>
#
# This file is part of Humphrey Videotheque.
#
# Humphrey Videotheque is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Humphrey Videotheque is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = """H. Turgut Uyar <uyar@tekir.org>"""
__docformat__ = "restructuredtext"


from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.CMFCore import permissions
from Products.CMFCore.utils import UniqueObject

import os
import urllib2
import zipfile
import re
import string
import imdb
import StringIO


IMDB_BASE_URL = "http://www.imdb.com"
IMDB_FILE = os.path.join(package_home(globals()), "imdb.zip")
IMDB_INFOSETS = ("main", "plot", "taglines", "keywords", "connections")
IMDB_SECTION_MAP = {
    "color":    ("Color-info", "color-info"),
    "genre":    ("Genres",     "genres"),
    "country":  ("Countries",  "countries"),
    "language": ("Languages",  "language"),
    }
IMDB_OPTION_PATTERN = '<A HREF="/List?.*?&&%s=.*?">(.*?)</A> \(([0-9]+)\)'
IMDB_KEYWORD_PATTERN = '<a href="/keyword/%s/">(.*?)</a></span>&nbsp;\(([0-9]+)\)'
#IMDB_KEYWORD_PATTERN = '<b>1 to [0-9]+</b> of <b>([0-9,]+)</b>'
NORMALIZATION_MAP = {
    "&nbsp;": " ",
    "&#160;": " ",
    }



def normalize(entry):
    normalized = entry.decode("iso8859-1")
    for k in NORMALIZATION_MAP.keys():
        normalized = normalized.replace(k, NORMALIZATION_MAP[k])
    return normalized.encode("utf-8")


def tokenize(entry):
    tokens = entry.split("::")
    if len(tokens) == 1:
        return tokens[0], None
    elif len(tokens) > 1:
        return tokens[:2]


def getOptionsFromImdb(option, threshold=1, useFiles=True):
    section, key = IMDB_SECTION_MAP[option]
    optionPath = "Sections/%s/types_all" % section
    if useFiles:
        imdb_archive = zipfile.ZipFile(IMDB_FILE)
        imdb_page = imdb_archive.read(optionPath)
        imdb_archive.close()
    else:
        imdb_url = "%s/%s" % (IMDB_BASE_URL, optionPath)
        imdb_page = urllib2.urlopen(imdb_url).read()
    matches = re.findall(IMDB_OPTION_PATTERN % key, imdb_page)
    options = [normalize(k) for k, c in matches if int(c) >= threshold]
    return tuple(options)


def getKeywordFromImdb(keyword, useFiles=True):
    optionPath = "Sections/Keywords/types_%s" % string.upper(keyword[0])
    if useFiles:
        imdb_archive = zipfile.ZipFile(IMDB_FILE)
        imdb_page = imdb_archive.read(optionPath)
        imdb_archive.close()
    else:
        imdb_url = "%s/%s" % (IMDB_BASE_URL, optionPath)
        imdb_page = urllib2.urlopen(imdb_url).read()
    match = re.search(IMDB_KEYWORD_PATTERN % keyword, imdb_page)
    if not match:
        return (None, 0)
    key = normalize(match.group(1))
    movies = int(match.group(2))
    return key, movies


class VideothequeTool(UniqueObject, SimpleItem, PropertyManager):
    """This tool provides methods for managing video collections."""

    id = "videotheque_tool"
    meta_type = "Videotheque Tool"
    plone_tool = 1

    manage_options = PropertyManager.manage_options

    security = ClassSecurityInfo()

    def __init__(self):
        self.manage_addProperty("imdbpy_access_system", "httpThin", "string")
        self.manage_addProperty("imdb_movie_threshold", 1000, "int")

        options = ("--not specified--", "PAL", "NTSC", "SECAM")
        self.manage_addProperty("tv_system_options", options, "lines")

        options = ("--not specified--", "Region 1", "Region 2")
        self.manage_addProperty("dvd_region_options", options, "lines")

        options = ("VHS", "Beta")
        self.manage_addProperty("tape_system_options", options, "lines")

        options = ("--not specified--", "16/9", "4/3")
        self.manage_addProperty("aspect_ratio_options", options, "lines")

        options = ("--not specified--", "DivX", "Xvid", "MPG", "AVI", "OGM",
                   "FLV", "MOV", "ASF", "WMV", "RM", "MKV", "ISO")
        self.manage_addProperty("video_codec_options", options, "lines")

        options = ("--not specified--", "MP3", "AC3", "PCM", "OGG", "DTS",
                   "FLAC", "M4A", "WMA")
        self.manage_addProperty("audio_codec_options", options, "lines")

        options = getOptionsFromImdb("color", self.imdb_movie_threshold)
        self.manage_addProperty("color_options", options, "lines")

        options = getOptionsFromImdb("genre", self.imdb_movie_threshold)
        self.manage_addProperty("genre_options", options, "lines")

        options = getOptionsFromImdb("country", self.imdb_movie_threshold)
        self.manage_addProperty("country_options", options, "lines")

        options = getOptionsFromImdb("language", self.imdb_movie_threshold)
        self.manage_addProperty("language_options", options, "lines")

        self.manage_addProperty("plot_keyword_options", "", "lines")

    security.declarePublic("getOptions")
    def getOptions(self, name):
        """Return the list of options for a given option name."""
        try:
            optionsProperty = getattr(self, name)
            options = [option for option in optionsProperty if option]
            return options
        except:
            return []

    security.declareProtected(permissions.View, "queryImdb")
    def queryImdb(self, title):
        class ImdbMovie:
            def __init__(self, movie):
                self.imdb_id = movie.movieID
                self.id = "imdb_%s" % self.imdb_id
                self.title = movie["title"]

            def title_or_id(self):
                return self.title

            def title_or_id_escaped(self):
                return self.title.replace("'", "\\\'")

            def Title(self):
                return self.title

            def absolute_url(self):
                return "%s/title/tt%s" % (IMDB_BASE_URL, self.imdb_id)

            def getType(self):
                return "Movie"

            def getIcon(self):
                return "movie.gif"

        connection = imdb.IMDb(self.imdbpy_access_system);
        results = []
        for movie in connection.search_movie(title):
            results.append(ImdbMovie(movie))
        return results

    security.declareProtected(permissions.View, "getMovieInfoFromImdb")
    def getMovieInfoFromImdb(self, movieid, akaCountries):
        connection = imdb.IMDb(self.imdbpy_access_system)
        movie = connection.get_movie(movieid, info=IMDB_INFOSETS)

        info = {}
        info["imdb_id"] = movie.movieID
        info["title"] = movie["canonical title"]

        if movie.has_key("year"):
            info["year"] = movie["year"]

        if movie.has_key("runtimes"):
            runtime = movie["runtimes"][0]
            for token in runtime.split(':'):
                try:
                    info["runtime"] = int(token)
                    break
                except ValueError:
                    continue

        if movie.has_key("color info"):
            colors = map(tokenize, movie["color info"])
            info["color"] = [k for k, c in colors]

        if movie.has_key("genres"):
            info["genres"] = movie["genres"]

        if movie.has_key("countries"):
            info["countries"] = movie["countries"]

        if movie.has_key("languages"):
            info["langs"] = movie["languages"]

        if movie.has_key("akas"):
            selected_akas = []
            akas = map(tokenize, movie["akas"])
            for aka, country in akas:
                if (aka in selected_akas) or (aka == info["title"]):
                    continue
                if country is None:
                    selected_akas.append(aka)
                else:
                    for ac in akaCountries:
                        if country.startswith("(%s" % ac):
                            selected_akas.append(aka)
            info["akas"] = selected_akas

        if movie.has_key("director"):
            info["directors"] = [p["name"] for p in movie["director"]]

        if movie.has_key("cast"):
            info["cast"] = [p["name"] for p in movie["cast"]]

        if movie.has_key("taglines"):
            info["tagline"] = movie["taglines"][0]

        if movie.has_key("plot outline"):
            info["plot_summary"] = movie["plot outline"]
        elif movie.has_key("plot"):
            info["plot_summary"] = tokenize(movie["plot"][0])[1]

        if movie.has_key("keywords"):
            keywords = []
            for keyword in movie["keywords"]:
                k, c = getKeywordFromImdb(keyword)
                if (k in self.plot_keyword_options) \
                        or (c >= self.imdb_movie_threshold):
                    keywords.append(k)
            info["plot_keywords"] = keywords

        if movie.has_key("cover"):
            cover_url = movie["cover"]
            cover_data = urllib2.urlopen(cover_url).read()
            info["poster"] = StringIO.StringIO(cover_data)

        if movie.has_key("rating"):
            info["imdb_rating"] = movie["rating"]

        connected_movies = []
        if movie.has_key("connections"):
            movie_connections = movie["connections"]
            if movie_connections.has_key("follows"):
                connected_movies.extend(movie_connections["follows"])
            if movie_connections.has_key("followed by"):
                connected_movies.extend(movie_connections["followed by"])
            info["connected_movies"] = [m.movieID for m in connected_movies]

        self.updateOptions(info)
        return info

    security.declarePrivate("updateOptions")
    def updateOptions(self, info):
        if info.has_key("color"):
            options = [o for o in info["color"]
                       if o not in self.color_options]
            self.color_options = self.color_options + tuple(options)

        if info.has_key("genres"):
            options = [o for o in info["genres"]
                       if o not in self.genre_options]
            self.genre_options = self.genre_options + tuple(options)

        if info.has_key("countries"):
            options = [o for o in info["countries"]
                       if o not in self.country_options]
            self.country_options = self.country_options + tuple(options)

        if info.has_key("langs"):
            options = [o for o in info["langs"]
                       if o not in self.language_options]
            self.language_options = self.language_options + tuple(options)

        if info.has_key("plot_keywords"):
            options = [o for o in info["plot_keywords"]
                       if o not in self.plot_keyword_options]
            self.plot_keyword_options = self.plot_keyword_options + tuple(options)

InitializeClass(VideothequeTool)
